import os
from infisical_sdk import InfisicalSDKClient


INFISICAL_HOST = "https://eu.infisical.com"
PYTHON_ENV = os.getenv("PYTHON_ENV")
INFISICAL_CLIENT_ID = os.getenv("INFISICAL_CLIENT_ID")
INFISICAL_SECRET = os.getenv("INFISICAL_SECRET")
INFISICAL_PROJECT = os.getenv("INFISICAL_PROJECT")

class SecretNotFoundError(Exception):
    """Se lanza cuando un secreto no se encuentra en Infisical."""
    pass

class InfisicalClientSingleton:
    _instance = None
    _client = None
    _host = None
    _client_id = None
    _client_secret = None
    _project_id = None
    _python_env = None

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
            cls._initialize_client()
        return cls._instance

    @staticmethod
    def _initialize_client():
        InfisicalClientSingleton._host = INFISICAL_HOST
        InfisicalClientSingleton._client_id = INFISICAL_CLIENT_ID
        InfisicalClientSingleton._client_secret = INFISICAL_SECRET
        InfisicalClientSingleton._project_id = INFISICAL_PROJECT
        InfisicalClientSingleton._python_env = PYTHON_ENV

        client = InfisicalSDKClient(host=InfisicalClientSingleton._host)
        client.auth.universal_auth.login(
            client_id=InfisicalClientSingleton._client_id,
            client_secret=InfisicalClientSingleton._client_secret)

        InfisicalClientSingleton._client = client

    @classmethod
    def get_client(cls):
        return cls._client


def get_secret(secret_name):
    """
    Obtiene el secreto especificado desde el cliente Infisical.

    Si el nombre del secreto comienza con `GLOBAL_`, se buscará en la raíz de carpetas de Infisical. 
    En caso contrario, se buscará en la carpeta correspondiente al nombre de la ETL que realiza la llamada.

    Args:
        secret_name (str): El nombre del secreto que se desea recuperar.

    Returns:
        str : El valor del secreto recuperado, en caso de encontrarlo.

    Excepciones:
    ------------
    SecretNotFoundError
        Si el secreto no se encuentra en Infisical, se lanza una excepción personalizada `SecretNotFoundError`.
    """
    if secret_name.startswith("GLOBAL_"):
        secret_path = "/"  
    else:
        execution_dir = os.getcwd() 
        parent_dir = os.path.dirname(execution_dir) 
        secret_folder = os.path.basename(parent_dir) 
        secret_path = f"/{secret_folder}"

    try:
        secret = InfisicalClientSingleton().get_client().secrets.get_secret_by_name(
            secret_name=secret_name,
            project_id=InfisicalClientSingleton._project_id,
            environment_slug=InfisicalClientSingleton._python_env,
            secret_path=secret_path
        )
    except Exception as e:
        raise SecretNotFoundError(f"El secreto '{secret_name}' no se encontró en '{secret_path}'.")

    return secret.secretValue
